MODULE m_vmtxcg
  !.....------------------------------------------------------------------
  !     fit spherical-harmonics expansion of exchange-correlation
  !     potential inside muffint-tin spheres and add it to coulomb
  !     potential
  !                                     c.l.fu and r.podloucky           *
  !     for the gradient correction. t.a. 1996.
  !.....------------------------------------------------------------------
  !     instead of vmtxcor.f: the different exchange-correlation
  !     potentials defined through the key icorr are called through
  !     the driver subroutine vxcallg.f, subroutines vectorized
  !     ** r.pentcheva 22.01.96
  !     *********************************************************
  !     angular mesh calculated on speacial gauss-legendre points
  !     in order to use orthogonality of lattice harmonics and
  !     avoid a least square fit
  !     ** r.pentcheva 04.03.96
  !     *********************************************************
  !
CONTAINS
  SUBROUTINE vmtxcg(&
       &                dimension,sphhar,atoms,&
       &                rho,xcpot,input,sym,&
       &                obsolete,&
       &                vxr,vr,rhmn,ichsmrg,&
       &                excr)

    USE m_lhglptg
    USE m_grdchlh
    USE m_mkgylm
    USE m_gaussp
    USE m_xcallg, ONLY : vxcallg,excallg

    USE m_types
    IMPLICIT NONE

    TYPE(t_xcpot),INTENT(IN)       :: xcpot
    TYPE(t_dimension),INTENT(IN)   :: dimension
    TYPE(t_obsolete),INTENT(IN)    :: obsolete
    TYPE(t_input),INTENT(IN)       :: input
    TYPE(t_sym),INTENT(IN)         :: sym
    TYPE(t_sphhar),INTENT(IN)      :: sphhar
    TYPE(t_atoms),INTENT(IN)       :: atoms
#ifdef CPP_MPI_OK
    include "mpif.h"
#endif
    !     ..
    !     .. Scalar Arguments ..
    INTEGER, INTENT(INOUT):: ichsmrg
    REAL,    INTENT(INOUT):: rhmn
    !     ..
    !     .. Array Arguments ..
    REAL,    INTENT (IN) :: rho(atoms%jmtd,0:sphhar%nlhd,atoms%ntypd,dimension%jspd)
    REAL,    INTENT (INOUT):: vxr(atoms%jmtd,0:sphhar%nlhd,atoms%ntypd,dimension%jspd)
    REAL,    INTENT (INOUT):: vr(atoms%jmtd,0:sphhar%nlhd,atoms%ntypd,dimension%jspd)
    REAL,    INTENT (OUT)  :: excr(atoms%jmtd,0:sphhar%nlhd,atoms%ntypd)
    !     ..
    !     .. Local Scalars ..
    INTEGER jr,js,k,lh,n,nd,ist,nsp,ixpm ,i,nat
    REAL    rhmnm,d_15,elh,vlh
    LOGICAL lwbc              ! if true, white-bird trick
    !     ..
    !     .. Local Arrays ..
    REAL vx(dimension%nspd,dimension%jspd),vxc(dimension%nspd,dimension%jspd),exc(dimension%nspd),rx(3,dimension%nspd)
    REAL vxl(dimension%nspd,dimension%jspd),vxcl(dimension%nspd,dimension%jspd),excl(dimension%nspd),divi
    REAL wt(dimension%nspd),rr2(atoms%jmtd),thet(dimension%nspd)
    REAL agr(dimension%nspd),agru(dimension%nspd),agrd(dimension%nspd),g2r(dimension%nspd),g2ru(dimension%nspd)
    REAL g2rd(dimension%nspd),gggr(dimension%nspd),gggru(dimension%nspd),gggrd(dimension%nspd)
    REAL grgru(dimension%nspd),grgrd(dimension%nspd),gzgr(dimension%nspd)
    REAL, ALLOCATABLE :: ylh(:,:,:),ylht(:,:,:),ylhtt(:,:,:)
    REAL, ALLOCATABLE :: ylhf(:,:,:),ylhff(:,:,:),ylhtf(:,:,:)
    REAL, ALLOCATABLE :: chlh(:,:,:),chlhdr(:,:,:),chlhdrr(:,:,:)
    REAL, ALLOCATABLE :: ch(:,:),chdr(:,:),chdt(:,:),chdf(:,:)
    REAL, ALLOCATABLE :: chdrr(:,:),chdtt(:,:),chdff(:,:),chdtf(:,:)
    REAL, ALLOCATABLE :: chdrt(:,:),chdrf(:,:)

    !locals for mpi
    integer :: ierr
    real,allocatable:: buffer(:)
    integer:: n_start,n_stride
    !     ..
    !     .. Intrinsic Functions ..
    INTRINSIC max,mod,min
    !ta+
    !.....------------------------------------------------------------------




    lwbc=.false.

    d_15 = 1.e-15
    ixpm = 1
    ist  = 1

    !     generates nspd points on a sherical shell with radius 1.0
    !     angular mesh equidistant in phi,
    !     theta are zeros of the legendre polynomials
    !
    CALL gaussp(&
         &            atoms%lmaxd,&
         &            rx,wt)

    nsp = dimension%nspd
    !
    !     generates the lattice harmonics on the angular mesh
    !
    ALLOCATE ( ylh(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd),ylht(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd),&
         &           ylhtt(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd),ylhf(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd),&
         &           ylhff(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd),ylhtf(dimension%nspd,0:sphhar%nlhd,sphhar%ntypsd) )

    CALL lhglptg(&
         &             sphhar,atoms,&
         &             rx,nsp,xcpot,sym,&
         &             ylh,thet,ylht,ylhtt,ylhf,ylhff,ylhtf)
    !
    !
    !     loop over topologically non-equivalent atoms
    !


#ifdef CPP_MPI_OK
    CALL MPI_COMM_RANK(MPI_COMM,mpi%irank,ierr)
    CALL MPI_COMM_SIZE(MPI_COMM,mpi%isize,ierr)

    n_start=mpi%irank+1
    n_stride=mpi%isize

    IF (mpi%irank>0) THEN
       vxr=0.0
       vr=0.0
    ENDIF
    excr=0.0

#else
    n_start=1
    n_stride=1
#endif


    !$OMP PARALLEL DO DEFAULT(SHARED)&
    !$OMP& PRIVATE(jr,js,k,lh,n,nd,nat,i,rhmnm,elh,vlh)&
    !$OMP& PRIVATE(vx,vxc,exc,vxl,vxcl,excl,divi,rr2)&
    !$OMP& PRIVATE(agr,agru,agrd,g2r,g2ru,g2rd,gggr,gggru,gggrd)&
    !$OMP& PRIVATE(grgru,grgrd,gzgr,chlh,chlhdr,chlhdrr,ch,chdr,chdt)&
    !$OMP& PRIVATE(chdf,chdrr,chdtt,chdff,chdtf,chdrt,chdrf)
    DO 200 n = 1,atoms%ntype
       ALLOCATE ( ch(dimension%nspd,dimension%jspd),chdr(dimension%nspd,dimension%jspd),chdt(dimension%nspd,dimension%jspd),&
            &       chdf(dimension%nspd,dimension%jspd),chdrr(dimension%nspd,dimension%jspd),chdtt(dimension%nspd,dimension%jspd),&
            &       chdff(dimension%nspd,dimension%jspd),chdtf(dimension%nspd,dimension%jspd),chdrt(dimension%nspd,dimension%jspd),&
            &       chdrf(dimension%nspd,dimension%jspd),chlh(atoms%jmtd,0:sphhar%nlhd,dimension%jspd),&
            &       chlhdr(atoms%jmtd,0:sphhar%nlhd,dimension%jspd),chlhdrr(atoms%jmtd,0:sphhar%nlhd,dimension%jspd))


       nat=sum(atoms%neq(:n-1))+1
       nd = atoms%ntypsy(nat)

       DO jr = 1,atoms%jri(n)
          rr2(jr) = 1.e0/ (atoms%rmsh(jr,n)*atoms%rmsh(jr,n))
       ENDDO

       DO lh = 0,sphhar%nlh(nd)

          !         calculates gradients of radial charge densities of l=> 0.
          !         rho*ylh/r**2 is charge density. chlh=rho/r**2.
          !         charge density=sum(chlh*ylh).
          !         chlhdr=d(chlh)/dr, chlhdrr=dd(chlh)/drr.

          DO js = 1,input%jspins

             DO jr = 1,atoms%jri(n)
                chlh(jr,lh,js) = rho(jr,lh,n,js)*rr2(jr)
             ENDDO

             IF (xcpot%igrd.GT.0) THEN 
                CALL grdchlh(&
                     &                     ixpm,ist,atoms%jri(n),atoms%dx(n),atoms%rmsh(1,n),&
                     &                     chlh(1,lh,js),obsolete%ndvgrd,&
                     &                     chlhdr(1,lh,js),chlhdrr(1,lh,js))
             ENDIF

          ENDDO ! js
       ENDDO   ! lh
       !
       !-->    loop over radial mesh 
       !
       DO 190 jr = 1,atoms%jri(n)
          !
          !         following are at points on jr-th sphere.
          !         initialize arrays
          !
          ch(:,:)    = 0.0     ! charge density
          chdr(:,:)  = 0.0     ! d(ch)/dr
          chdt(:,:)  = 0.0     ! d(ch)/dtheta
          chdf(:,:)  = 0.0     ! d(ch)/dfai
          chdrr(:,:) = 0.0     ! dd(ch)/drr
          chdtt(:,:) = 0.0     ! dd(ch)/dtt
          chdff(:,:) = 0.0     ! dd(ch)/dff
          chdtf(:,:) = 0.0     ! dd(ch)/dtf
          chdrt(:,:) = 0.0     ! d(d(ch)/dr)dt
          chdrf(:,:) = 0.0     ! d(d(ch)/dr)df

          DO js = 1,input%jspins
             DO lh = 0,sphhar%nlh(nd)

                !             generate the densities on an angular mesh

                DO k = 1,nsp
                   ch(k,js) = ch(k,js) + ylh(k,lh,nd)*chlh(jr,lh,js)
                ENDDO

                IF (xcpot%igrd.GT.0) THEN
                   ! 
                   DO k = 1,nsp
                      chdr(k,js) =chdr(k,js)+ ylh(k,lh,nd)*chlhdr(jr,lh,js)
                      chdrr(k,js)=chdrr(k,js)+ylh(k,lh,nd)*chlhdrr(jr,lh,js)
                   ENDDO

                   DO k = 1,nsp
                      chdrt(k,js)=chdrt(k,js)+ylht(k,lh,nd)*chlhdr(jr,lh,js)
                      chdrf(k,js)=chdrf(k,js)+ylhf(k,lh,nd)*chlhdr(jr,lh,js)
                      chdt(k,js) =chdt(k,js) +ylht(k,lh,nd)*chlh(jr,lh,js)
                      chdf(k,js) =chdf(k,js) +ylhf(k,lh,nd)*chlh(jr,lh,js)
                      chdtt(k,js)=chdtt(k,js)+ylhtt(k,lh,nd)*chlh(jr,lh,js)
                      chdff(k,js)=chdff(k,js)+ylhff(k,lh,nd)*chlh(jr,lh,js)
                      chdtf(k,js)=chdtf(k,js)+ylhtf(k,lh,nd)*chlh(jr,lh,js)
                   ENDDO


                ENDIF

             ENDDO ! lh
          ENDDO   ! js

          IF (xcpot%igrd.GT.0) THEN
             CALL mkgylm(&
                  &                  input%jspins,atoms%rmsh(jr,n),thet,nsp,dimension%nspd,dimension%jspd,ch,chdr,&
                  &                  chdt,chdf,chdrr,chdtt,chdff,chdtf,chdrt,chdrf,&
                  &                  agr,agru,agrd,g2r,g2ru,g2rd,gggr,gggru,gggrd,&
                  &                  grgru,grgrd,gzgr)!keep
          ELSE
             agr(:)   = 0.0 ; agru(:)  = 0.0 ; agrd(:)  = 0.0 
             g2r(:)   = 0.0 ; g2ru(:)  = 0.0 ; g2rd(:)  = 0.0 
             gggr(:)  = 0.0 ; gggru(:) = 0.0 ; gggrd(:) = 0.0 
             grgru(:) = 0.0 ; grgrd(:) = 0.0 ; gzgr(:)  = 0.0 
          ENDIF

          !
          !         rhmnm: rho_minimum_muffin-tin..

          rhmnm=10.e+10

          DO js=1,input%jspins
             DO i=1,nsp
                ch(i,js) = max(ch(i,js),d_15)
                rhmnm = min(rhmnm,ch(i,js))
             ENDDO
          ENDDO

          IF (rhmnm.LT.rhmn) THEN
             rhmn = rhmnm
             ichsmrg = 1
          ENDIF

          IF (rhmn.LT.obsolete%chng) THEN
             WRITE (6,'(/'' rhmn.lt.obsolete%chng in vmtxc. rhmn,obsolete%chng='',&
                  &        2d9.2)') rhmn,obsolete%chng
             !             CALL juDFT_error("vmtxcg: rhmn.lt.chng",calledby="vmtxcg")
          ENDIF

          !
          !         calculate the ex.-cor. potential

          IF (mod(jr,1000).eq.0)&
               &      WRITE (6,'(/'' 9999ic,kr,stars%ig,js,nsp,iwb='',5i5,l3/&
               &            '' ch''/(10d15.7))') xcpot%icorr,input%krla,xcpot%igrd,input%jspins,&
               &            nsp,lwbc,((ch(k,js),k=1,nsp),js=1,input%jspins)

#ifdef CPP_600
#endif
          CALL vxcallg(&
               &                 xcpot%icorr,lwbc,input%jspins,nsp,nsp,ch,agr,agru,agrd,&
               &                 g2r,g2ru,g2rd,gggr,gggru,gggrd,gzgr,&
               &                 vx,vxc)!keep
#ifdef CPP_600
          IF (atoms%jri(n) < 600) THEN
          ELSE
             CALL vxcallg(&
                  &                 -1,lwbc,input%jspins,nsp,ch,agr,agru,agrd,&
                  &                 g2r,g2ru,g2rd,gggr,gggru,gggrd,gzgr,&
                  &                 vxl,vxcl)!keep
             divi = 1.0 / (atoms%rmsh(atoms%jri(n),n) - atoms%rmsh(1,n))
             vxc(:,:) = ( vxcl(:,:) * ( atoms%rmsh(atoms%jri(n),n) - atoms%rmsh(jr,n) ) +&
                  &                  vxc(:,:) * ( atoms%rmsh(jr,n) - atoms%rmsh(1,n) ) ) * divi
          ENDIF
#endif

          IF (mod(jr,1000).eq.0)&
               &              WRITE (6,'(/'' 999vxc''/(10d15.7))')&
               &                    ((vxc(k,js),k=1,nsp),js=1,input%jspins)


          !         now determine the corresponding potential number

          DO js = 1,input%jspins
             !
             !           multiplicate vx/vxc with the weights of the k-points
             !
             DO k = 1,nsp
                vx (k,js) = vx (k,js)*wt(k)
                vxc(k,js) = vxc(k,js)*wt(k)
             ENDDO

             IF (mod(jr,1500).EQ.0)&
                  &        WRITE (6,'('' 999wt''/(10d15.7))') (wt(k),k=1,nsp)
             IF (mod(jr,1500).EQ.0)&
                  &        WRITE (6,'('' 999vxc''/(10d15.7))') (vxc(k,js),k=1,nsp)

             DO lh = 0,sphhar%nlh(nd)
                !
                ! --->        determine the corresponding potential number
                !c            through gauss integration
                !
                vlh=dot_product(vxc(:nsp,js),ylh(:nsp,lh,nd))
                vr(jr,lh,n,js) = vr(jr,lh,n,js) + vlh

                ! --->        add to the given potential

                IF (mod(jr,1500).EQ.0) THEN
                   WRITE(6,'('' 999lh,js,jr,n,vlh='',4i5,d15.7)')&
                        &            lh,js,jr,n,vlh
                   WRITE(6,'('' 9ylh''/(10d15.7))') (ylh(k,lh,nd),k=1,nsp)
                ENDIF

                vlh=dot_product(vx(:nsp,js),ylh(:nsp,lh,nd))
             
                vxr(jr,lh,n,js) = vxr(jr,lh,n,js) + vlh

             ENDDO ! lh
          ENDDO   ! js


          IF (input%total) then
             !
             !           calculate the ex.-cor energy density
             !
             IF (mod(jr,2500).EQ.0)&
                  &        WRITE (6,'(/'' 9999ic,kr,stars%ig,js,nsp='',5i5/&
                  &               '' ch''/(10d15.7))') xcpot%icorr,input%krla,xcpot%igrd,input%jspins,&
                  &               nsp,((ch(k,js),k=1,nsp),js=1,input%jspins)

#ifdef CPP_600
#endif
             CALL excallg(xcpot%icorr,lwbc,input%jspins,nsp,&
                  &                   ch,agr,agru,agrd,g2r,g2ru,g2rd,&
                  &                   gggr,gggru,gggrd,gzgr,&
                  &                   exc)!keep
#ifdef CPP_600
             IF (atoms%jri(n) < 600) THEN
             ELSE
                CALL excallg(-1,lwbc,input%jspins,nsp,&
                     &                   ch,agr,agru,agrd,g2r,g2ru,g2rd,&
                     &                   gggr,gggru,gggrd,gzgr,&
                     &                   excl)!keep

                divi = 1.0 / ( atoms%rmsh(atoms%jri(n),n) - atoms%rmsh(1,n))
                exc(:) = ( excl(:) * ( atoms%rmsh(atoms%jri(n),n) - atoms%rmsh(jr,n) ) +&
                     &                exc(:) * ( atoms%rmsh(jr,n) - atoms%rmsh(1,n) ) ) * divi
             ENDIF
#endif


             IF (mod(jr,10000).EQ.0)&
                  &        WRITE (6,'(/'' 999exc''/(10d15.7))') (exc(k),k=1,nsp)

          ENDIF

          !         now determine the corresponding energy density number
          !
          !         multiplicate exc with the weights of the k-points
          !
          DO k = 1,nsp
             exc(k) = exc(k)*wt(k)
          ENDDO

          DO lh = 0,sphhar%nlh(nd)
             !
             !           determine the corresponding potential number through gauss
             !           integration
             !
             elh = 0.0
             DO k = 1,nsp
                elh = elh + exc(k)*ylh(k,lh,nd)
             ENDDO
             excr(jr,lh,n) =  elh

          ENDDO

190    ENDDO

       !        WRITE(6,'(/'' n='',i3/'' 9999vr''/(10d15.7))') n,
       !     &   (((vr(jr,lh,n,js),jr=1,jri(n),100),lh=0,ntypsy(nat)),js=1,jspins)
       !        WRITE(6,'(/'' 9999excr''/(10d15.7))')
       !     &   ((excr(jr,lh,n),jr=1,jri(n),100),lh=0,ntypsy(nat))

       DEALLOCATE (ch,chdr,chdt,chdf,chdrr,chdtt,chdff,chdtf,chdrt,chdrf)
       DEALLOCATE (chlh,chlhdr,chlhdrr)


200 ENDDO
    !$OMP END PARALLEL DO
#ifdef CPP_MPI_OK
    if (mpi%irank==0) allocate(buffer(size(vxr))

    CALL MPI_REDUCE(vxr,buffer,size(vxr),CPP_MPI_REAL,MPI_SUM,0,&
         &                                mpi,ierr)
    IF (mpi%irank==0) vxr=reshape(buffer,shape(vxr))

    CALL MPI_REDUCE(vr,buffer,size(vxr),CPP_MPI_REAL,MPI_SUM,0,&
         &                                mpi,ierr)
    IF (mpi%irank==0) vr=reshape(buffer,shape(vr))

    CALL MPI_REDUCE(excr,buffer,size(excr),CPP_MPI_REAL,MPI_SUM,0,&
         &                                mpi,ierr)
    IF (mpi%irank==0) excr=reshape(buffer(:size(excr)),shape(excr))
    IF (mpi%irank==0) deallocate(buffer)
#endif
    !

    RETURN
  END SUBROUTINE vmtxcg
END MODULE m_vmtxcg
